"""Disabled as of 2.1.6 but it does seem to work.Except with CD
"""
# TODO Fix dearpygui module not found
# import dearpygui.dearpygui as dpg
#
# import sys
# this = sys.modules[__name__]
#
# for i in dir(dpg):
#     setattr(this, i, getattr(dpg, i))
